Param (
        [Alias("n")][string]$TaskName,
        [Alias("s")][string]$StackName,
        [Alias("r")][string]$Region,
        [Alias("w")][string]$WaitHandle,
        [Alias("i")][string]$InstanceName,
        [Alias("h")][string]$HostName,
        [Alias("j")][string]$NodeJSONPath,
        [Alias("l")][string]$ChefLogFile
      )
 
###############################################################################
##
function Run-VMSetup([hashtable]$SetupData)
{
  Set-EC2ConfigPlugInState -PlugInName "AWS.EC2.Windows.CloudWatch.PlugIn" -State Enabled

  Restart-Service "ec2config"

  Sleep -Seconds 60  # give the service some time to init CloudWatch plugin

  Run-Process -Name "cfn-init"  -Arguments "-v -s $($SetupData.StackName) -r $($SetupData.InstanceName) --region $($SetupData.Region)"
  Run-Process -Name "chef-solo" -Arguments "-j $($SetupData.NodeJSONPath) -L $($SetupData.ChefLogFile) -l info"

  cfn-signal $SetupData.WaitHandle
  Log-Message "Done!"
}

###############################################################################
##
function Log-Message([string]$Msg,[switch]$PassThru) 
{
  "[{0}] $Msg" -f (Get-Date -Format "MM-dd-yyyy HH:mm:ss.fffff") | Add-Content $logFile -Force -ea SilentlyContinue
  if ($PassThru) {
    Write-Output $Msg
  }
}

###############################################################################
##
function Set-EC2ConfigPlugInState([string]$PlugInName,[ValidateSet("Enabled","Disabled")][string]$State) 
{
  $cfgFile = Join-Path $env:ProgramFiles -ChildPath "Amazon\Ec2ConfigService\Settings\Config.xml"
  Log-Message "Setting PlugIn '$PlugInName' state to '$State'"

  $xml = [xml](Get-Content $cfgFile)
  $xml.get_DocumentElement().Plugins.Plugin | ?{$_.name -ieq $PlugInName} | %{$_.State = $State}
  $xml.Save($cfgFile)
}

###############################################################################
##
function Run-Process([string]$Name,[string]$Arguments)
{
  Log-Message "Executing : $Name"
  Log-Message "Arguments : $Arguments"
  $process = Start-Process $Name -ArgumentList $Arguments.Split(' ') -Wait -PassThru
  if ($process.ExitCode -ne 0) {
    throw "Process $Name exit code : $($process.ExitCode). See CloudWatch Logs for details."  
  }
} 

###############################################################################
##
function Calculate-StartUpDateTime([System.Globalization.CultureInfo]$Culture = [System.Globalization.CultureInfo]::CurrentCulture,
                                   [int]$DelaySeconds = 60)
{
  $timePattern = "HH:mm:ss" 
  $datePattern = @{ 'd' = "dd/MM/yyyy"  
                    'm' = "MM/dd/yyyy"  
                    'y' = "yyyy/MM/dd" }.Item($Culture.DateTimeFormat.ShortDatePattern.Substring(0,1).ToLower())
  
  $whenStart = (get-date) + (New-TimeSpan -Seconds $DelaySeconds)
  @{ date = "{0:$datePattern}" -f $whenStart 
     time = "{0:$timePattern}" -f $whenStart }
} 

###############################################################################
##                          SCRIPT ENTRY POINT
###############################################################################
##
try
{
  $thisScript = $MyInvocation.MyCommand.Path 
  $dataFile   = $thisScript + ".data.txt"
  $logFile    = Join-Path $env:ProgramFiles -ChildPath "Amazon\Ec2ConfigService\Logs\esriVMSetupLog.txt"

  Log-Message "DEBUG: Executing script '$thisScript' on machine '$env:COMPUTERNAME'"

  if ($HostName) 
  {
    if ($HostName.Length -gt 15) {
      Log-Message "WARNING: host name '$HostName' is too long. Must be 3-15 chars long."
      $HostName = $HostName.Substring(0,15)
      Log-Message "ACTION : cutting the host name down to '$HostName'"
    }

    if ($env:COMPUTERNAME -ine $HostName) {
      Log-Message "-------------- Rename Machine --------------" -PassThru
      Log-Message "Current name : $($env:COMPUTERNAME)" -PassThru
      Log-Message "New name     : $HostName" -PassThru
      Set-EC2ConfigPlugInState -PlugInName "Ec2HandleUserData" -State Enabled
      Set-EC2ConfigPlugInState -PlugInName "AWS.EC2.Windows.CloudWatch.PlugIn" -State Enabled
      Log-Message "-------------- Reboot Machine --------------" -PassThru
      Rename-Computer -NewName $HostName -Restart -Verbose -Force
      exit
    }
    Log-Message "Host name : $HostName"
  } 

  if ($TaskName)
  {
    Log-Message "----------------- Set task -----------------" -PassThru
    @("StackName    = $StackName"
      "Region       = $Region"
      "WaitHandle   = $WaitHandle"
      "InstanceName = $InstanceName"
      "NodeJSONPath = $NodeJSONPath"
      "ChefLogFile  = $ChefLogFile"
      ) | Set-Content $dataFile

    $runTask = "powershell.exe -ExecutionPolicy Unrestricted -File '$thisScript'"
    $startup = Calculate-StartUpDateTime
    schtasks /create /sc once /tn "'$TaskName'" /tr "$runTask" /st $startup.time /sd $startup.date /F /RU SYSTEM /RL HIGHEST 
  } 
  else 
  {
    Log-Message "----------------- VM Setup -----------------"
    $setupData  = Get-Content $dataFile | %{$_.Replace("\\","\").Replace("\","\\")} | Out-String | ConvertFrom-StringData
    $WaitHandle = $setupData.WaitHandle
    Run-VMSetup $setupData
  } 
}
catch
{
  Log-Message "ERROR: $($_.Exception.Message)" -PassThru
  Start-Sleep -Seconds 300
  cfn-signal -e 1 -r "$($_.Exception.Message)" "$WaitHandle"
}

## EOf